require "defines"
require "util"
hf = require "scripts/helper-functions"	-- contains helper functions
um = require "scripts/umarker"			-- contains utilitymarker code
bb = require "scripts/bridgebuilder"	-- contains bridgebuilder
aStar = require "scripts/a-star"			-- A* pathfinding functions

-- EVENTS
--   ontick = 0,
--  onguiclick = 1,
--  onentitydied = 2,
--  onpickedupitem = 3,
--  onbuiltentity = 4,
--  onsectorscanned = 5,
--  onplayermineditem = 6,
--  onputitem = 7,
--  onshiplandingstart = 8,
--  onpreplayermineditem = 9,
--  onchunkgenerated = 10,
--  onplayercrafteditem = 11







----------------------------------------------------------------------------------------------------------
-- function: 		game.oninit
-- description:		called directly after starting a new game / running a game with the mod the first time 
-- parameter:		none
-- return:			none
----------------------------------------------------------------------------------------------------------
game.oninit(function()
	hf.giveItems()
	InitDataTables()	
end)

----------------------------------------------------------------------------------------------------------
-- function: 		game.onload
-- description:		called after loading a game 
-- parameter:		none
-- return:			none
----------------------------------------------------------------------------------------------------------
game.onload(function()	
	InitDataTables()
end)

----------------------------------------------------------------------------------------------------------
-- function: 		game.onevent(defines.events.onentitydied, function(event)
-- description:		called after an entity died 
-- parameter:		none
-- return:			event.entity contains the entity
----------------------------------------------------------------------------------------------------------
game.onevent(defines.events.onentitydied, function(event)
	if event.entity.name == "utilitymarker-2" then glob.utilityMarker.count = glob.utilityMarker.count - 1 end
end)

----------------------------------------------------------------------------------------------------------
-- function: 		game.onevent(defines.events.onplayermineditem, function(event)
-- description:		called after player mined an entity 
-- parameter:		none
-- return:			event.itemstack contains the item
----------------------------------------------------------------------------------------------------------
game.onevent(defines.events.onplayermineditem, function(event)
	if event.itemstack.name == "utilitymarker-2" then glob.utilityMarker.count = glob.utilityMarker.count - 1 end
end)

----------------------------------------------------------------------------------------------------------
-- function: 		game.onevent(defines.events.onbuiltentity, function(event)
-- description:		called after player build something 
-- parameter:		none
-- return:			event.createdentity contains the entity
----------------------------------------------------------------------------------------------------------
game.onevent(defines.events.onbuiltentity, function(event)
	if glob.utilityMarker.markerUsed == false then
		if event.createdentity.name == "utilitymarker-2" then	
			um.onUtilityMarkerBuilt(event.createdentity)
		end
	else
		if event.createdentity.name == "utilitymarker-2" then
			game.player.print("You already placed 2 markers. Assign a task first!")
			game.player.character.insert{name = event.createdentity.name, count = 1}
			event.createdentity.destroy()
		end
	end
end)

----------------------------------------------------------------------------------------------------------
-- function: 		game.onevent(defines.events.ontick, function(event)
-- description:		event is raised every tick ( normally every 1/60 sec) 
-- parameter:		none
-- return:			none
----------------------------------------------------------------------------------------------------------
game.onevent(defines.events.ontick, function(event)

	if glob.utilityMarker.count < 0 then glob.utilityMarker.count = 0 end

	if bb.startcalc == true then
		if glob.utilityMarker.marker[2].valid == true then glob.utilityMarker.marker[2].destroy() end
		if glob.utilityMarker.marker[1].valid == true then glob.utilityMarker.marker[1].destroy() end
		
		for i=1,2 do
			currentPos = aStar.calcPath(glob.utilityMarker.bridgeMode.startPos,glob.utilityMarker.bridgeMode.endPos, glob.utilityMarker.bridgeMode.mode)
			if currentPos ~= nil then
				if currentPos.x ~= nil then
					glob.utilityMarker.bridgeMode.startPos.x = currentPos.x
					glob.utilityMarker.bridgeMode.startPos.y = currentPos.y
					table.insert(glob.utilityMarker.bridgeMode.overlay, game.createentity{name ="umOverlay", position = currentPos,force=game.player.force})
				else
					while #currentPos > 0 do
						hf.createOverlay(currentPos[#currentPos], currentPos[#currentPos])
						table.remove(currentPos)
					end
					game.player.print("done")
					bb.startcalc = false
					break
				end
			else
				game.player.print("error")
				bb.startcalc = false
				break
			end
		end
	end
end)

----------------------------------------------------------------------------------------------------------
-- function: 		game.onevent(defines.events.onguiclick, function(event)
-- description:		event is raised if player clicked a custom UI-button
-- parameter:		none
-- return:			event.element contains the UI-element
----------------------------------------------------------------------------------------------------------
game.onevent(defines.events.onguiclick, function(event)
	local prefix = hf.getButtonPrefix(event.element.name)

	if prefix == "um" then um.mainMenuButtonClicked(event.element.name)				-- utilitymarker main-menu
	elseif prefix == "bb" then bb.bbButtonClicked(event.element.name)				-- bridgebuilder buttons
	end
end)

----------------------------------------------------------------------------------------------------------
-- function: 		InitDataTables
-- description:		initializes the needed tables/vars for the mod 
-- parameter:		none
-- return:			none
----------------------------------------------------------------------------------------------------------
function InitDataTables()
	if glob.utilityMarker == nil then
		glob.utilityMarker = {}												-- Namespace
			glob.utilityMarker.count = 0										-- # of placed utilMarker
			glob.utilityMarker.marker = {}										-- contains the placed marker entities
				glob.utilityMarker.minPos = {x = 0, y = 0}
				glob.utilityMarker.maxPos = {x = 0, y = 0}
				glob.utilityMarker.area = 0
				glob.utilityMarker.area_x = 0
				glob.utilityMarker.area_y = 0
			glob.utilityMarker.markerUsed = false								-- flag to indicate if the GUI is open

			glob.utilityMarker.overlay = {}										-- basic table for overlay
			glob.utilityMarker.overlay.size = 0										-- # of overlay-elements
			glob.utilityMarker.overlay.elements = {}								-- contains overlay-entities

			glob.utilityMarker.bridgeMode = {}									-- basic table for bridge builder functionality
				glob.utilityMarker.bridgeMode.startPos = {x = 0, y = 0}
				glob.utilityMarker.bridgeMode.endPos = {x = 0, y = 0}
				glob.utilityMarker.bridgeMode.mode = "avoidAll"
				glob.utilityMarker.bridgeMode.overlay = {}
	end
end