module(..., package.seeall)



----------------------------------------------------------------------------------------------------------
-- function: 		giveItems()
-- description:		gives the player some usefull stuff on start; for debugging  
-- parameter:		none
-- return:			none
----------------------------------------------------------------------------------------------------------
function giveItems()
	game.player.character.insert{name="iron-plate", count=8}
	game.player.character.insert{name="basic-bullet-magazine", count=190}
	game.player.character.insert{name="utilitymarker-2", count=64}
	game.player.character.insert{name="wooden-chest", count=64}
	game.player.character.insert{name="small-electric-pole", count=32}
	game.player.character.insert{name="basic-inserter", count=64}
	game.player.character.insert{name="solar-panel", count=54}
	game.player.character.insert{name="basic-transport-belt", count=128}
	game.player.character.insert{name="steam-engine", count=16}
	game.player.character.insert{name="boiler", count=32}
	game.player.character.insert{name="gun-turret", count=32}
	game.player.character.insert{name="lab", count=8}
	game.player.character.insert{name="pipe", count=64}
	game.player.character.insert{name="basic-mining-drill", count=32}
	game.player.character.insert{name="basic-transport-belt-to-ground", count=32}
	game.player.character.insert{name="pipe-to-ground", count=32}
	game.player.character.insert{name="basic-splitter", count=32}
	game.player.character.insert{name="straight-rail", count=32}
	game.player.character.insert{name="curved-rail", count=32}
	game.player.character.insert{name="iron-chest", count=32}
	game.player.character.insert{name="fast-inserter", count=32}
	game.player.character.insert{name="flame-thrower", count=1}
	game.player.character.insert{name="flame-thrower-ammo", count=100}
	game.player.character.insert{name="submachine-gun", count=1}
	game.player.character.insert{name="heavy-armor", count=2}
	game.player.character.insert{name="land-mine", count=16}
	game.player.character.insert{name="assembling-machine-1", count=16}
	game.player.character.insert{name="assembling-machine-2", count=16}
	game.player.character.insert{name="assembling-machine-3", count=16}
	game.player.character.insert{name="diesel-locomotive", count=16}
	game.player.character.insert{name="coal", count=128}
	game.player.character.insert{name="car", count=2}
end

----------------------------------------------------------------------------------------------------------
-- function: 		getButtonPrefix(buttonName)
-- description:		extracts the prefix of a name (first 2 letters) 
-- parameter:		name of the button
-- return:			prefix
----------------------------------------------------------------------------------------------------------
function getButtonPrefix(buttonName)
	local prefix = string.sub(buttonName,1,2)
	return prefix
end

----------------------------------------------------------------------------------------------------------
-- function: 		countPlayerStructures(minPos, maxPos, mode)
-- description:		used to count different kind of entities
-- parameter:		minPos, maxPos
--					mode ("trees"; "structures")
-- return:			number of entities
----------------------------------------------------------------------------------------------------------
function countEntities(minPos, maxPos, mode)
	local entities = {}
	if mode == "trees" then
		entities = game.findentitiesfiltered{area = {minPos, maxPos}, type="tree"}
		return #entities
	elseif mode == "structures" then
		entities = game.findentities{minPos, maxPos}
		local k = 0
		for _, entity in ipairs(entities) do
			if (entity.hasflag("player-creation")) and not (entity.force.equals(game.forces.enemy)) then
				if (entity.name ~= "bpOverlay") and (entity.name ~= "umOverlay") and (entity.name ~= "utilitymarker-2") then
					k = k + 1
				end
			end
		end
		return k
	else
		game.player.print("Wrong number of arguments. Mode was not passed")
		return nil
	end
end

----------------------------------------------------------------------------------------------------------
-- function: 		createOverlay(minPos, maxPos)
-- description:		creates overlay graphics
-- parameter:		minPos, maxPos
-- return:			none
----------------------------------------------------------------------------------------------------------
function createOverlay(minPos, maxPos)
	for x = 0, (maxPos.x - minPos.x) do
		for y = 0, (maxPos.y - minPos.y) do
			glob.utilityMarker.overlay.elements[glob.utilityMarker.overlay.size + 1] = game.createentity{name ="umOverlay", position ={minPos.x + x, minPos.y + y},force=game.player.force}
			glob.utilityMarker.overlay.size = glob.utilityMarker.overlay.size + 1
		end
	end
end

----------------------------------------------------------------------------------------------------------
-- function: 		destroyOverlay()
-- description:		used to count different kind of entities
-- parameter:		minPos, maxPos
--					mode ("trees"; "structures")
-- return:			number of entities
----------------------------------------------------------------------------------------------------------
function destroyOverlay()
	while (glob.utilityMarker.overlay.size > 0) do
		local entity = glob.utilityMarker.overlay.elements[glob.utilityMarker.overlay.size]
		 if entity.valid then entity.destroy() end
		 glob.utilityMarker.overlay.size = glob.utilityMarker.overlay.size - 1
	end
end









function create_smoke(position)
	game.createentity{name = "smoke", position = position}
	game.createentity{name = "smoke", position = position}
	game.createentity{name = "smoke", position = position}
	game.createentity{name = "smoke", position = position}
	game.createentity{name = "smoke", position = position}
end


function remove_nanobots(mode)
	local neededNanobots = 0
	local actualnanobots = game.player.character.getinventory(1).getitemcount("nanobots") + game.player.character.getinventory(2).getitemcount("nanobots")
	if mode == "load" then
		neededNanobots = glob.blueprints.savetable.nanobots
	elseif mode == "clear" then
		neededNanobots = glob.blueprints.utilitymarker.nanobots
	end

	if actualnanobots >= neededNanobots then
		game.player.removeitem{name="nanobots",count = neededNanobots}
	end
end


function counttypes()
	if ent_types == nil then
		ent_types = {}
	else
		for k,_ in ipairs(ent_types) do
  			ent_types[k] = nil
		end
	end
		
	local exists = false
	local j = 0
	
	for _, entity in ipairs(glob.blueprints.savetable.entities) do
		for i = 1, #ent_types ,2 do
			if ent_types[i] == entity.name then
				ent_types[i+1] = ent_types[i+1] + 1
				exists = true
				break
			end
			j = i
		end
		if exists == false then
			table.insert(ent_types , entity.name)
			table.insert(ent_types , 1)
		end
		exists = false
	end
	
	return ent_types -- returns table; table-format = {entity-one-name, entity-one-count, entity-two-name, entity-two-count, ...}
end



